//
// Copyright (c) 2012 Linaro Limited
//
// All rights reserved. This program and the accompanying materials
// are made available under the terms of the MIT License which accompanies
// this distribution, and is available at
// http://www.opensource.org/licenses/mit-license.php
//
// Contributors:
//     Jesse Barker - original implementation.
//
#ifndef GL_IF_H_
#define GL_IF_H_
// Inclusion abstraction to provide project specific interface headers for
// whatever flavor of OpenGL(|ES) is appropriate.  For core libmatrix, this
// is GLEW.
#include "gl-headers.h"
#endif // GL_IF_H_
